# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class MD:
    qr = []

    def __init__(self, filename):
        self.lines = open(filename, "rb").readlines()

    def set_qr_url(self, url):
        if 'http' == url[:4]:
            url = '"' + url + '"'
        self.qr = [
            '<section>\n',
            '  <h1>Thank you</h1>\n',
            '  <p>You can find this slide at <div id="qr-url">-</div> or scan this QR code</p>\n',
            '  <canvas id="qr-code"></canvas>\n',
            '  <script src="http://liriansu.com/static/js/qr.min.js"></script>\n',
            '  <script>\n',
            '    qr.canvas({\n',
            '      canvas: document.getElementById("qr-code"),\n',
            '      level : "H",\n',
            '      size  : 8,\n',
            '      value : %s\n' % url,
            '    });\n',
            '    document.getElementById("qr-url").innerHTML = %s;\n' % url,
            '  </script>\n',
            '</section>\n',
        ]

    def dump_sections(self):
        # Add section tag
        res = ['<section data-markdown>\n']
        for l in self.lines:
            if self.head(l) > 0:
                res += ['</section>\n', '<section data-markdown>\n']
            res += [l]
        res += ['</section>\n']

        # Remove empty section
        i = len(res) - 1
        while i >= 0:
            if res[i] == '<section data-markdown>\n' and res[i+1] == '</section>\n':
                res = res[:i] + res[i+2:]
            else:
                i -= 1

        # Indent
        indents, indent, start = [], 0, -1
        for i in range(len(res)):
            head = self.head(res[i])
            if (head == 1 and start > -1) or (head == 2 and indent > 2):
                indents += [[start, i-1]]
            if head > 0:
                indent = head
            if head == 1:
                start = -1
            elif head == 2 or (head == 3 and start == -1):
                start = i-1
        if start > 0:
            indents += [[start, len(res)]]

        for i in reversed(indents):
            for j in range(i[0], i[1]):
                res[j] = '  ' + res[j]
            res = res[:i[0]] + ['<section>\n'] + res[i[0]:i[1]] + ['</section>\n'] + res[i[1]:]

        # Add qr code section
        res += self.qr

        return res

    def head(self, line):
        head, idx = 0, 0
        while line.find('#', idx) > -1:
            head, idx = head + 1, line.find('#', idx) + 1
        return head



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import argparse
import sys
from os.path import basename, isfile, splitext

from md2reveal.md import MD
from md2reveal.reveal import Reveal


def execute(argv=None):
    if argv is None:
        argv = sys.argv[1:] or ['-h']
    parser = argparse.ArgumentParser(description='convert markdown file to reveal.js html page')
    parser.add_argument('markdown_file', help='the markdown file')
    parser.add_argument('-m', '--theme', help='the reveal.js theme')
    parser.add_argument('-t', '--title', help='the html title')
    parser.add_argument('-o', '--outfile', help='output file name')
    parser.add_argument('-q', '--qrcode', help='add qrcode as last section', action='store_true')
    parser.add_argument('-l', '--qrurl', help='the qrcode url')
    args = parser.parse_args(argv)
    md = args.markdown_file
    theme = args.theme or 'black'
    themes = ['beige', 'black', 'blood', 'league', 'moon', 'night', 'serif', 'simple', 'sky', 'solarized', 'white']
    if theme not in themes:
        print('{} is not a valid theme:\n{}'.format(theme, '\n'.join(themes)))
        exit(1)
    elif not isfile(md):
        print('{} is not a file.'.format(md))
        exit(1)
    else:
        out = args.outfile or (basename(splitext(md)[0]) + ".html")
        print('Generating [{}]'.format(out))
        m = MD(md)
        if args.qrcode:
            m.set_qr_url(args.qrurl or 'window.location.href')
        r = Reveal(m.dump_sections(), theme)
        r.generate(out, args.title or basename(splitext(md)[0]))
        print('Finish, you can find it at [{}]'.format(out))


if __name__ == '__main__':
    execute()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from os.path import dirname, join


class Reveal:
    def __init__(self, lines, theme):
        lines = map(lambda l: '        ' + l if l.strip() else '\n', lines)
        self.html = open(join(dirname(__file__), 'template.html'), 'rb').read().replace(
            '---theme---', theme).replace('---section---', '\n{}'.format(''.join(lines)))

    def generate(self, filename, title):
        open(filename, 'wb').write(self.html.replace('---title---', title))
